@@ -110,13 +110,13 @@ class QuoteData(object):
 
     @assert_quote_init
     @lru_cache(maxsize=None, typed=True)
-    def _fetch_data(self, filename, date_columns=None):
+    def _fetch_data(self, filename, index_col=None, date_columns=None):
         """获取数据
 
         参数:
             filename: 数据文件名
-            dtype: 字段类型
-            parse_dates: 日期字段
+            index_col：索引列
+            date_columns: 日期字段
 
         返回值: DataFrame
 
@@ -139,7 +139,7 @@ class QuoteData(object):
                 parse_dates = [date_columns]
             else:
                 raise ValueError('date_columns is error.')
-            result = pd.read_csv(data_file, dtype={'code': str}, parse_dates=parse_dates)
+            result = pd.read_csv(data_file, index_col=index_col, dtype={'code': str}, parse_dates=parse_dates)
         elif extname == 'txt':
             with open(data_file, 'r') as f:
                 result = f.readlines()
@@ -258,7 +258,7 @@ class QuoteData(object):
 
         """
         year_minimum = 1990
-        start = datetime.datetime(2005,1,1).date()
+        start = datetime.datetime(2005, 1, 1).date()
         if period in ['day', 'daily', '1d']:
             trade_list = self.get_trade_days(end_date=date, count=count)
             if trade_list:
@@ -385,7 +385,7 @@ class QuoteData(object):
             np_data[:, 1:] = df[cols_px + cols_vm].values
             self.trace('exec', time.time() - proc_start)
 
-            operate_params = list()
+            operate_params = []
             for code, group in df.groupby(by='code'):
                 if len(group) == 0 or code not in xrxd.index:
                     continue
@@ -549,7 +549,7 @@ class QuoteData(object):
                 if not se.empty:
                     df.loc[se.index, cols_fn] = se.tolist() if len(cols_fn) > 1 else se
 
-        #指标名称-变更前缀或转换大小写
+        # 指标名称-变更前缀或转换大小写
         cols_maps = dict()
         for x in df.columns:
             if x not in cols_techansi:
@@ -639,6 +639,175 @@ class QuoteData(object):
         result = [datetime.date.fromtimestamp(x.astype(datetime.datetime) / 1e9) for x in result]
         return result
 
+
+    @assert_quote_init
+    def get_block_info(self, block=None, types=None) -> pd.DataFrame:
+        """获取板块列表
+
+        参数:
+            block: 板块代码或者板块代码的 list
+            types: 板块类型；类型 字符串 list；默认值 None 获取所有板块
+                   可选值: 'index', 'industry', 'concept' 等；
+
+        返回值:
+            DataFrame 对象：
+                block（索引）：板块代码
+                name：板块名称
+                date：类型 datetime.date；行业与概念板块表示启用日期；指数板块表示最后更新日期
+                type：证券类型；stock（股票）, index（指数），etf（ETF基金），fja（分级A），fjb（分级B）
+
+        """
+        block = formatter_list(block)
+        types = formatter_list(types)
+
+        filename = 'quote-block-map.csv'
+        df = self._fetch_data(filename, date_columns='date')
+        if df is None:
+            raise Exception('fetch data from %s failed.' % filename)
+        if types:
+            df = df[df['type'].isin(types)]
+        if block:
+            df = df[df['block'].isin(block)]
+        result = df.copy()
+        result.set_index('block', inplace=True)
+        return result
+
+
+    @assert_quote_init
+    def get_industries(self) -> pd.DataFrame:
+        """获取行业列表
+
+        说明：
+            本方法等同 get_block_info(types='industry')
+            返回值移除索引名称，变更字段名 date 为 start_date
+
+        """
+        result = self.get_block_info(types='industry')
+        result.index.name = None
+        result.rename(columns={'date': 'start_date'}, inplace=True)
+        return result
+
+
+    @assert_quote_init
+    def get_concepts(self) -> pd.DataFrame:
+        """获取概念列表
+
+        说明：
+            本方法等同 get_block_info(types='concept')
+            返回值移除索引名称，变更字段名 date 为 start_date
+
+        """
+        result = self.get_block_info(types='concept')
+        result.index.name = None
+        result.rename(columns={'date': 'start_date'}, inplace=True)
+        return result
+
+
+    @assert_quote_init
+    def get_block_securities(self, block, types=None) -> list:
+        """获取板块成份股
+
+        参数:
+            block: 板块代码，字符串
+            types: 板块类型；类型 字符串 list；默认值 None
+                   可选值: 'index', 'industry', 'concept' 等；
+
+        返回值:
+            证券代码列表
+
+        """
+
+        if not block:
+            raise ValueError('block need be provided.')
+        if not type(block) is str:
+            raise ValueError('block is invalid.')
+        types = formatter_list(types)
+
+        filename = 'quote-block-data.csv'
+        df = self._fetch_data(filename)
+        if df is None:
+            raise Exception('fetch data from %s failed.' % filename)
+        if types:
+            df = df[df['type'].isin(types)]
+        if block:
+            df = df[df['block'] == block]
+        result = df['security'].tolist()
+        return result
+
+
+    @assert_quote_init
+    def get_index_weights(self, index_code) -> list:
+        """获取指数成份股权重
+
+        参数:
+            index_code: 指数代码，字符串
+
+        返回值:
+            DataFrame 对象：
+                code（索引）：证券代码
+                date：更新日期
+                weight：权重值
+                display_name：证券名称
+
+        """
+
+        if not index_code:
+            raise ValueError('index_code need be provided.')
+        if not type(index_code) is str:
+            raise ValueError('index_code is invalid.')
+
+        filename = 'quote-block-data.csv'
+        df = self._fetch_data(filename)
+        if df is None:
+            raise Exception('fetch data from %s failed.' % filename)
+        df = df[(df['block'] == index_code) & (df['type'] == 'index')]
+        result = df.copy()
+        result.drop(columns=['block', 'type'], inplace=True)
+        result.set_index('security', inplace=True)
+        result.index.name = 'code'
+
+        df = self.get_block_info(index_code, types='index')
+        date = df['date'][0] if not df.empty else None
+        result.insert(0, 'date', date)
+
+        df = self.get_security_info()
+        result['display_name'] = df['display_name']
+        return result
+
+
+    @assert_quote_init
+    def get_index_stocks(self, index_code) -> list:
+        """获取指数成份股
+
+        说明：
+            本方法等同 get_block_securities(<block>, types='index')
+
+        """
+        return self.get_block_securities(index_code, types='index')
+
+
+    @assert_quote_init
+    def get_industry_stocks(self, industry_code) -> list:
+        """获取行业成份股
+
+        说明：
+            本方法等同 get_block_securities(<block>, types='industry')
+
+        """
+        return self.get_block_securities(industry_code, types='industry')
+
+
+    @assert_quote_init
+    def get_concept_stocks(self, concept_code) -> list:
+        """获取概念成份股
+
+        说明：
+            本方法等同 get_block_securities(<block>, types='concept')
+
+        """
+        return self.get_block_securities(concept_code, types='concept')
+
+
     @assert_quote_init
     def get_security_info(self, security=None, fields=None, types=None) -> pd.DataFrame:
         """获取证券信息数据
@@ -651,6 +820,7 @@ class QuoteData(object):
 
         返回值:
             DataFrame 对象，包含 fields 指定的字段：
+                code（索引）：证券代码
                 display_name：中文名称
                 name：缩写简称
                 start_date：上市日期；类型 datetime.date
@@ -677,7 +847,7 @@ class QuoteData(object):
             cols_all = list(df.columns)
             cols_drop = [x for x in cols_all if x not in cols_retain]
             df.drop(columns=cols_drop, inplace=True)
-        result = result.append(df, ignore_index=True)
+        result = pd.concat([result, df], sort=False, ignore_index=True)
         result.set_index('code', inplace=True)
         return result
 
@@ -760,7 +930,7 @@ class QuoteData(object):
                 cols_all = list(df.columns)
                 cols_drop = [x for x in cols_all if x not in cols_retain]
                 df.drop(columns=cols_drop, inplace=True)
-            result = result.append(df, ignore_index=True)
+            result = pd.concat([result, df], sort=False, ignore_index=True)
         return result
 
     @assert_quote_init
@@ -857,7 +1027,7 @@ class QuoteData(object):
                         if c in sec_list:
                             sec_list.remove(c)
                 # self.execute_rehabilitation(df, fq, fq_vol, columns=cols_retain, inplace=True)
-                result = result.append(df, ignore_index=True)
+                result = pd.concat([result, df], sort=False, ignore_index=True)
                 if check_security_count:
                     if not sec_list:
                         break
@@ -876,7 +1046,8 @@ class QuoteData(object):
             dt_start = np.datetime64(
                 datetime.datetime.fromtimestamp(time.mktime(start_date))) if start_date else None
             dt_end = np.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date)))
-            append_paused = 'paused' in fields or skip_paused
+            all_tdays = self.get_trade_days()
+            curr_tdays = self.get_trade_days(start_date=start_date, end_date=end_date)
             for c in security:
                 p = c.find('.')
                 if p < 0:
@@ -889,17 +1060,51 @@ class QuoteData(object):
                 if df is None:
                     continue
                 df = df.copy()
-                if append_paused:
-                    if period_type == 'day':
-                        df['paused'] = df.apply(lambda x: 1 if x['volume'] == 0 and x['money'] == 0 else 0, axis=1)
-                    elif period_type == 'mn1':
-                        c_period = 'day'
-                        c_file = os.path.join(c_period, exc_code, exc_code + '_' + c_period.upper()
-                                              + '_' + sec_code + '.csv')
-                        c_df = self._fetch_data(c_file, date_columns='datetime')
-                        c_skip_days = c_df[(c_df['volume'] == 0) & (c_df['money'] == 0)]['datetime'].tolist()
-                        df['paused'] = df['datetime'].apply(
-                            lambda x: 1 if x.astype('datetime64[D]').isin(c_skip_days) else 0)
+                if period_type == 'day':
+                    df['paused'] = df.apply(lambda x: 1 if x['volume'] == 0 and x['money'] == 0 else 0, axis=1)
+                elif period_type == 'mn1':
+                    df['date'] = pd.to_datetime(df['datetime'].dt.date)
+                    df_sum = df[['date', 'volume', 'money']].groupby('date').sum()
+                    paused_days = df_sum[(df_sum['volume'] == 0) & (df_sum['money'] == 0)].index.tolist()
+                    df['paused'] = df.apply(lambda x: 1 if x['date'] in paused_days else 0, axis=1)
+                    df.drop(columns=['date'], inplace=True)
+                if not skip_paused:
+                    paused_days = sorted(list(set(curr_tdays).difference(set(df['datetime'].dt.date.tolist()))))
+                    c_date = c_price = None
+                    for curr_date in paused_days:
+                        prev_date = None
+                        i = all_tdays.index(curr_date) - 1
+                        while i >= 0:
+                            if all_tdays[i] not in paused_days:
+                                prev_date = all_tdays[i]
+                                break
+                            i -= 1
+                        if not prev_date:
+                            continue
+                        if c_date != prev_date:
+                            c_values = df[df['datetime'].dt.date == prev_date]['close'].values
+                            if len(c_values) > 0 and not np.isnan(c_values[-1]):
+                                c_date = prev_date
+                                c_price = c_values[-1]
+                        if prev_date == c_date and c_price:
+                            x = {'datetime': datetime.datetime.combine(curr_date, datetime.time.min), 'open': c_price,
+                                 'close': c_price, 'high': c_price, 'low': c_price, 'volume': 0, 'money': 0,
+                                 'paused': 1}
+                            y = []
+                            if period_type == 'day':
+                                y.append(x.copy())
+                            elif period_type == 'mn1':
+                                for i in range(240):
+                                    t = 571 + i
+                                    if i >= 120:
+                                        t += 90
+                                    x['datetime'] = datetime.datetime.combine(curr_date,
+                                                                              datetime.time.min) + datetime.timedelta(
+                                        minutes=t)
+                                    y.append(x.copy())
+                            if y:
+                                df = pd.concat([df, pd.DataFrame(y, columns=df.columns)], ignore_index=False)
+                                df.sort_values('datetime', inplace=True)
                 if skip_paused:
                     df = df[df['paused'] == 0]
                 if not prep_count:
@@ -908,7 +1113,7 @@ class QuoteData(object):
                     df = df[df['datetime'] <= dt_end][-prep_count:]
                 df.insert(loc=0, column='code', value=sec_code + '.' + exc_code)
                 # self.execute_rehabilitation(df, fq, fq_vol, columns=cols_retain, inplace=True)
-                result = result.append(df, ignore_index=True)
+                result = pd.concat([result, df], sort=False, ignore_index=True)
 
         # 多周期分组聚合
         agg_need = True
@@ -950,6 +1155,20 @@ class QuoteData(object):
 
         return result
 
+    @assert_quote_init
+    def get_price(self, security, period=None, frequency=None, fre_step=None, fq='pre', fq_vol=False, fields=None,
+                        start_date=None, end_date=None, count=None, skip_paused=False):
+        """获取K线数据
+
+        说明：
+            本方法为 get_quote_kdata 的别名，其中周期支持 period, frequency, fre_step
+
+        """
+
+        period = period if period else frequency if frequency else fre_step if fre_step else 'day'
+        return self.get_quote_kdata(security, period=period, fq=fq, fq_vol=fq_vol, fields=fields, start_date=start_date
+                                    , end_date=end_date, count=count, skip_paused=skip_paused)
+
     def get_technical_analysis(self, security, indications, period='day', fq='pre', fq_vol=False, fields=None,
                                start_date=None, end_date=None, count=None, skip_paused=False, keep_quote=False):
         """获取技术指标
@@ -1009,10 +1228,11 @@ class QuoteData(object):
             dt_start = np.datetime64(
                 datetime.datetime.fromtimestamp(time.mktime(start_date))) if start_date else None
             dt_end = np.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date)))
-            result = result.append(df[(df['datetime'] >= dt_start) & (df['datetime'] <= dt_end)], ignore_index=True)
+            result = pd.concat([result, df[(df['datetime'] >= dt_start) & (df['datetime'] <= dt_end)]],
+                               sort=False, ignore_index=True)
         else:
             for x, y in df.groupby(by='code'):
-                result = result.append(y.iloc[-count:], ignore_index=True)
+                result = pd.concat([result, y.iloc[-count:]], sort=False, ignore_index=True)
         result.reset_index(drop=True, inplace=True)
 
         # 是否保留行情字段
