@@ -166,7 +166,9 @@ class ArithmeticOperator(object):
         return stat
 
     def update(self, data):
-        self._data = self._data.append(data[['open', 'close', 'high', 'low', 'volume', 'money']])
+        df = pd.DataFrame([data.values], columns=data.index)
+        df = df[['open', 'close', 'high', 'low', 'volume', 'money']]
+        self._data = pd.concat([self._data, df], sort=False)
         self._index = 0
 
     def ref(self, x, n=0):
@@ -178,6 +180,7 @@ class ArithmeticOperator(object):
         if label:
             if n >= len(self._data):
                 n = -1
+            # print(type(self._data), self._data, n, label)
             r = self._data.iloc[-n - 1][label]
             if label == 'volume':
                 r = r / 100
